@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Copyright © 2011-2012 VideoLAN
- * $Id: d14e0f7ac9f4b846cbcf0750840093a5e5e621f9 $
+ * $Id: d085719058a906f775faece0f69c4c3411184a9f $
  *
  * Authors: Ludovic Fauvet <etix@l0cal.com>
  *
@@ -26,11 +26,12 @@
 #include <QPainterPath>
 #include <QBitmap>
 #include <QFontMetrics>
+#include <QDesktopWidget>
 
 #define TIP_HEIGHT 5
 
 TimeTooltip::TimeTooltip( QWidget *parent ) :
-    QWidget( parent )
+    QWidget( parent ), mInitialized( false )
 {
     setWindowFlags( Qt::Window                  |
                     Qt::WindowStaysOnTopHint    |
@@ -41,7 +42,7 @@ TimeTooltip::TimeTooltip( QWidget *parent ) :
     // a paintEvent occurs. This should save some CPU cycles.
     setAttribute( Qt::WA_OpaquePaintEvent );
 
-#if defined( Q_WS_WIN ) || defined( Q_WS_PM )
+#if defined( Q_OS_WIN ) || defined( Q_OS_OS2 )
     /*
     - This attribute is required on Windows and OS/2 to avoid focus stealing of other windows.
     - When set on Linux the TimeTooltip appears behind the FSController in fullscreen.
@@ -51,16 +52,13 @@ TimeTooltip::TimeTooltip( QWidget *parent ) :
 
     // Inherit from the system default font size -5
     mFont = QFont( "Verdana", qMax( qApp->font().pointSize() - 5, 7 ) );
-    mPreviousMetricsWidth = 0;
-
-    // Set default text
-    setText( "00:00:00", "" );
+    mTipX = -1;
 
     // By default the widget is unintialized and should not be displayed
     resize( 0, 0 );
 }
 
-void TimeTooltip::buildPath()
+void TimeTooltip::adjustPosition()
 {
     if( mDisplayedText.isEmpty() )
     {
@@ -68,21 +66,42 @@ void TimeTooltip::buildPath()
         return;
     }
 
-    QFontMetrics metrics( mFont );
-
     // Get the bounding box required to print the text and add some padding
-    QRect textbox = metrics.boundingRect( mDisplayedText ).adjusted( -2, -2, 2, 2 );
-
-    if ( mPreviousMetricsWidth == textbox.width() )
-        return; //same width == same path
-    else
-        mPreviousMetricsWidth = textbox.width();
-
-    mBox = QRect( 0, 0, textbox.width(), textbox.height() );
+    QFontMetrics metrics( mFont );
+    QRect textbox = metrics.boundingRect( mDisplayedText );
+    textbox.adjust( -2, -2, 2, 2 );
+    textbox.moveTo( 0, 0 );
 
     // Resize the widget to fit our needs
-    resize( mBox.width() + 1, mBox.height() + TIP_HEIGHT + 1 );
+    QSize size( textbox.width() + 1, textbox.height() + TIP_HEIGHT + 1 );
 
+    // The desired label position is just above the target
+    QPoint position( mTarget.x() - size.width() / 2,
+        mTarget.y() - size.height() + TIP_HEIGHT / 2 );
+
+    // Keep the tooltip on the same screen if possible
+    QRect screen = QApplication::desktop()->screenGeometry( mTarget );
+    position.setX( qMax( screen.left(), qMin( position.x(),
+        screen.left() + screen.width() - size.width() ) ) );
+    position.setY( qMax( screen.top(), qMin( position.y(),
+        screen.top() + screen.height() - size.height() ) ) );
+
+    move( position );
+
+    int tipX = mTarget.x() - position.x();
+    if( mBox != textbox || mTipX != tipX )
+    {
+        mBox = textbox;
+        mTipX = tipX;
+
+        resize( size );
+        buildPath();
+        setMask( mMask );
+    }
+}
+
+void TimeTooltip::buildPath()
+{
     // Prepare the painter path for future use so
     // we only have to generate the text at runtime.
 
@@ -91,12 +110,10 @@ void TimeTooltip::buildPath()
     mPainterPath.addRect( mBox );
 
     // Draw the tip
-    int center = mBox.width() / 2;
     QPolygon polygon;
-    polygon << QPoint( center - 3,   mBox.height() )
-            << QPoint( center,       mBox.height() + TIP_HEIGHT )
-            << QPoint( center + 3,   mBox.height() );
-
+    polygon << QPoint( qMax( 0, mTipX - 3 ), mBox.height() )
+            << QPoint( mTipX, mBox.height() + TIP_HEIGHT )
+            << QPoint( qMin( mTipX + 3, mBox.width() ), mBox.height() );
     mPainterPath.addPolygon( polygon );
 
     // Store the simplified version of the path
@@ -107,33 +124,40 @@ void TimeTooltip::buildPath()
     mMask = QBitmap( size() );
     QPainter painter( &mMask );
     painter.fillRect( mMask.rect(), Qt::white );
-    painter.setPen( QColor( 0, 0, 0 ) );
-    painter.setBrush( QColor( 0, 0, 0 ) );
+    painter.setPen( Qt::black );
+    painter.setBrush( Qt::black );
     painter.drawPath( mPainterPath );
     painter.end();
-
-    setMask( mMask );
 }
 
-void TimeTooltip::setText( const QString& time, const QString& text )
+void TimeTooltip::setTip( const QPoint& target, const QString& time, const QString& text )
 {
+    mInitialized = true;
     mDisplayedText = time;
     if ( !text.isEmpty() )
         mDisplayedText.append( " - " ).append( text );
 
-    if ( time.length() != mTime.length() || mText != text )
-        buildPath();
+    if( mTarget != target || time.length() != mTime.length() || mText != text )
+    {
+        mTarget = target;
+        mTime = time;
+        mText = text;
+        adjustPosition();
+    }
 
-    mTime = time;
-    mText = text;
     update();
-    raise();
 }
 
 void TimeTooltip::show()
 {
     setVisible( true );
-    raise();
+#ifdef Q_OS_OS2
+    // Bring a tooltip on the top
+    // Without this, tooltip does not appear on fullscreen
+    // from the second fullscreen state change
+    if( mInitialized )
+        QWidget::raise();
+#endif
 }
 
 void TimeTooltip::paintEvent( QPaintEvent * )
@@ -149,5 +173,3 @@ void TimeTooltip::paintEvent( QPaintEvent * )
     p.setPen( QPen( qApp->palette().text(), 1 ) );
     p.drawText( mBox, Qt::AlignCenter, mDisplayedText );
 }
-
-#undef TIP_HEIGHT
