@@ -1,8 +1,8 @@
 /*****************************************************************************
- * intf.c: Generic lua interface functions
+ * osd.c: Generic lua interface functions
  *****************************************************************************
  * Copyright (C) 2007-2008 the VideoLAN team
- * $Id: 9843a0dcbc2d74795748007dcaa2f4d7a58eb191 $
+ * $Id: 2d3ee1f94a2a2ac5123854c2b50d425875ee8505 $
  *
  * Authors: Antoine Cellerier <dionoea at videolan tod org>
  *
@@ -33,10 +33,7 @@
 #endif
 
 #include <vlc_vout.h>
-#include <vlc_osd.h>
-
-#include <lua.h>        /* Low level lua C API */
-#include <lauxlib.h>    /* Higher level C API */
+#include <vlc_vout_osd.h>
 
 #include "../vlc.h"
 #include "../libs.h"
@@ -218,55 +215,6 @@ static int vlclua_spu_channel_clear( lua_State *L )
     return 0;
 }
 
-static int vlclua_menu_show( lua_State *L )
-{
-    vlc_object_t *p_this = vlclua_get_this( L );
-    osd_MenuShow( p_this );
-    return 0;
-}
-
-static int vlclua_menu_hide( lua_State *L )
-{
-    vlc_object_t *p_this = vlclua_get_this( L );
-    osd_MenuHide( p_this );
-    return 0;
-}
-
-static int vlclua_menu_prev( lua_State *L )
-{
-    vlc_object_t *p_this = vlclua_get_this( L );
-    osd_MenuPrev( p_this );
-    return 0;
-}
-
-static int vlclua_menu_next( lua_State *L )
-{
-    vlc_object_t *p_this = vlclua_get_this( L );
-    osd_MenuNext( p_this );
-    return 0;
-}
-
-static int vlclua_menu_up( lua_State *L )
-{
-    vlc_object_t *p_this = vlclua_get_this( L );
-    osd_MenuUp( p_this );
-    return 0;
-}
-
-static int vlclua_menu_down( lua_State *L )
-{
-    vlc_object_t *p_this = vlclua_get_this( L );
-    osd_MenuDown( p_this );
-    return 0;
-}
-
-static int vlclua_menu_activate( lua_State *L )
-{
-    vlc_object_t *p_this = vlclua_get_this( L );
-    osd_MenuActivate( p_this );
-    return 0;
-}
-
 /*****************************************************************************
  *
  *****************************************************************************/
@@ -279,23 +227,9 @@ static const luaL_Reg vlclua_osd_reg[] = {
     { NULL, NULL }
 };
 
-static const luaL_Reg vlclua_menu_reg[] = {
-    { "show", vlclua_menu_show },
-    { "hide", vlclua_menu_hide },
-    { "prev", vlclua_menu_prev },
-    { "next", vlclua_menu_next },
-    { "up", vlclua_menu_up },
-    { "down", vlclua_menu_down },
-    { "activate", vlclua_menu_activate },
-    { NULL, NULL }
-};
-
 void luaopen_osd( lua_State *L )
 {
     lua_newtable( L );
     luaL_register( L, NULL, vlclua_osd_reg );
-    lua_newtable( L );
-    luaL_register( L, NULL, vlclua_menu_reg );
-    lua_setfield( L, -2, "menu" );
     lua_setfield( L, -2, "osd" );
 }
